package javawrapper;

import junit.framework.*;
import java.io.FileWriter;
import java.io.File;

/**
 * Tests the WorkerBase class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class TestWorkerBase extends TestCase {

  /** Test object */
  private WorkerTester m_oWorkerBaseTester = null;

  /**
   * Does set up by creating the test object.
   * @throws Exception Won't.
   */
  protected void setUp() throws Exception {
    super.setUp();
    m_oWorkerBaseTester = new WorkerTester();
  }

  /**
   * Does tear down.
   * @throws Exception Won't.
   */
  protected void tearDown() throws Exception {
    m_oWorkerBaseTester = null;
    super.tearDown();
  }

  /**
   * Tests the MakeSureAllAreProportions methods.
   */
  public void testMakeSureAllAreProportions() {

    //Normal processing
    Float fVal;
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector("", "", "", iNumElements,
                                          ModelVector.FLOAT);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {
      //Case - all are proportions and are applied to all
      fVal = new Float(0.11);
      for (i = 0; i < iNumElements; i++) {
        fVal = new Float(fVal.floatValue() + 0.1);
        p_oData.GetValue().add(i, fVal);
        p_bAppliesTo[i] = true;
      }
      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);

      //Case - all are proportions and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);

      //Case - some are applied and some non-proportions exist in unapplied
      //buckets
      p_bAppliesTo[2] = false;
      p_oData.GetValue().remove(2);
      p_oData.GetValue().add(2, new Float(2.3));
      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllAreProportions testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureAllAreProportions normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches non-proportion values that are negative
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData = null;
      p_oData = new ModelVector("", "", "", iNumElements, ModelVector.FLOAT);
      fVal = new Float(0.11);
      for (i = 0; i < iNumElements; i++) {
        fVal = new Float(fVal.floatValue() + 0.1);
        p_oData.GetValue().add(i, fVal);
        p_bAppliesTo[i] = true;
      }

      p_oData.GetValue().remove(2);
      p_oData.GetValue().add(2, new Float( -2.3));

      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);
      fail("MakeSureAllAreProportions failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches non-proportion values that are greater than 1
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData = null;
      p_oData = new ModelVector("", "", "", iNumElements, ModelVector.FLOAT);
      fVal = new Float(0.11);
      for (i = 0; i < iNumElements; i++) {
        fVal = new Float(fVal.floatValue() + 0.1);
        p_oData.GetValue().add(i, fVal);
        p_bAppliesTo[i] = true;
      }

      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Float(2.3));

      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);
      fail("MakeSureAllAreProportions failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllAreProportions exception processing testing succeeded.");
  }

  /**
   * Testing to make sure there are no negative values in a float array.
   */
  public void testMakeSureAllNonNegative() {
    int iNumElements = 4, i;
    Float[] p_fData = new Float[iNumElements];
    String sName = "Float Test";
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-negative and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0);
      p_fData[1] = new Float(2.3);
      p_fData[2] = new Float(0.74);
      p_fData[3] = new Float(5.88);
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_fData, sName, p_bAppliesTo);

      //Case - all are non-negative and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_fData, sName, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_fData[0] = new Float( -0.3);
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_fData, sName, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllNonNegative testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureAllNonNegative normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0);
      p_fData[1] = new Float( -2.3);
      p_fData[2] = new Float(0.74);
      p_fData[3] = new Float(5.88);

      m_oWorkerBaseTester.MakeSureAllNonNegative(p_fData, sName, p_bAppliesTo);
      fail("MakeSureAllNonNegative normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllNonNegative exception processing testing succeeded.");
  }

  /**
   * Testing for MakeSureAllNonZero in ModelVectors
   */
  public void testMakeSureAllNonZero() {
    //Normal processing
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector(
        "ModelVector testing for MakeSureAllNonZero", "", "", iNumElements,
        ModelVector.INTEGER);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-zero and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().add(0, new Integer(5));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(-74));
      p_oData.GetValue().add(3, new Integer(588));
      m_oWorkerBaseTester.MakeSureAllNonZero(p_oData, p_bAppliesTo);

      //Case - all are non-zero and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllNonZero(p_oData, p_bAppliesTo);

      //Case - some are applied and some zeroes exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Integer( 0 ));
      m_oWorkerBaseTester.MakeSureAllNonZero(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllNonZero testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureAllNonZero normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches zero values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(2));
      p_oData.GetValue().add(1, new Integer(0));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer( -588));

      m_oWorkerBaseTester.MakeSureAllNonZero(p_oData, p_bAppliesTo);
      fail("MakeSureAllNonZero normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllNonZero exception processing testing succeeded.");

  }

  /**
   * Testing to make sure there are no negative values in an integer array.
   */
  public void testMakeSureAllNonNegative1() {
    int iNumElements = 4, i;
    Integer[] p_iData = new Integer[iNumElements];
    String sName = "Integer test";
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-negative and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer(0);
      p_iData[1] = new Integer(2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(588);
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_iData, sName, p_bAppliesTo);

      //Case - all are non-negative and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_iData, sName, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_iData[0] = new Integer( -3);
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_iData, sName, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllNonNegative testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureAllNonNegative normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer(0);
      p_iData[1] = new Integer( -2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(588);

      m_oWorkerBaseTester.MakeSureAllNonNegative(p_iData, sName, p_bAppliesTo);
      fail("MakeSureAllNonNegative normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllNonNegative exception processing testing succeeded.");

  }

  /**
   * Testing for non-negatives in ModelVectors
   */
  public void testMakeSureAllNonNegative2() {
    //Normal processing
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector(
        "ModelVector testing for MakeSureAllNonNegative", "", "", iNumElements,
        ModelVector.INTEGER);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-negative and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(588));
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

      //Case - all are non-negative and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Integer( -3));
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllNonNegative testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureAllNonNegative normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer( -588));

      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);
      fail("MakeSureAllNonNegative normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllNonNegative exception processing testing succeeded.");

  }

  /**
   * Testing to make sure all values in a Float array are greater than 0.
   */
  public void testMakeSureAllPositive() {
    int iNumElements = 4, i;
    Float[] p_fData = new Float[iNumElements];
    String sName = "Float Test";
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are positive and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0.66);
      p_fData[1] = new Float(2.3);
      p_fData[2] = new Float(0.74);
      p_fData[3] = new Float(5.88);
      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);

      //Case - all are positive and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_fData[0] = new Float( -0.3);
      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllPositive testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureAllPositive normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0.66);
      p_fData[1] = new Float( -2.3);
      p_fData[2] = new Float(0.74);
      p_fData[3] = new Float(5.88);

      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches 0 values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0.66);
      p_fData[1] = new Float(2.3);
      p_fData[2] = new Float(0);
      p_fData[3] = new Float(5.88);

      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }
    System.out.println(
        "MakeSureAllPositive exception processing testing succeeded.");
  }

  /**
   * Testing to make sure all values in a vector are between two bounds.
   */
  public void testMakeSureAllAreBounded() {
    //Normal processing
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector(
        "ModelVector testing for MakeSureAllAreBounded", "", "", iNumElements,
        ModelVector.INTEGER);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are between the bounds and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(100));
      m_oWorkerBaseTester.MakeSureAllAreBounded(p_oData, p_bAppliesTo, 0, 100);

      //Case - all are between the bounds and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);

      //Case - some are applied and some out-of-bounds exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Integer( -3));
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllAreBounded testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureAllAreBounded normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches below-lower-bound values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(1));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer( -588));

      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);
      fail("MakeSureAllAreBounded failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches above-upper-bound values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(588));

      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);
      fail("MakeSureAllAreBounded normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllAreBounded exception processing testing succeeded.");
  }

  /**
   * Tests to make sure all values in an Integer array are greater than 0.
   */
  public void testMakeSureAllPositive1() {
    int iNumElements = 4, i;
    Integer[] p_iData = new Integer[iNumElements];
    String sName = "Integer Test";
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are positive and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer(66);
      p_iData[1] = new Integer(2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(588);
      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);

      //Case - all are positive and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_iData[0] = new Integer( -3);
      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllPositive testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureAllPositive normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer( -66);
      p_iData[1] = new Integer(2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(588);

      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches 0 values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer(66);
      p_iData[1] = new Integer(2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(0);

      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }
    System.out.println(
        "MakeSureAllPositive exception processing testing succeeded.");

  }

  /**
   * Tests MakeSureAllPositive methods.
   */
  public void testMakeSureAllPositive2() {
    //Normal processing
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector(
        "ModelVector testing for MakeSureAllPositive", "", "", iNumElements,
        ModelVector.INTEGER);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-negative and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().add(0, new Integer(1));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(588));
      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);

      //Case - all are non-negative and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Integer( -3));
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllPositive testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureAllPositive normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(1));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer( -588));

      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches 0 values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(588));

      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllPositive exception processing testing succeeded.");

  }

  /**
   * Tests to make sure a ModelFloat value is a proportion.
   */
  public void testMakeSureIsProportion() {
    ModelFloat fValue = new ModelFloat( (float) 0.45,
                                       "ModelFloat proportion test", "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureIsProportion(fValue);
    }
    catch (ModelException oErr) {
      fail("MakeSureIsProportion testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureIsProportion normal processing testing succeeded.");

    //Exception processing
    try {
      fValue.SetValue( (float) - 0.23);
      m_oWorkerBaseTester.MakeSureIsProportion(fValue);
      fail("MakeSureIsProportion failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      fValue.SetValue( (float) 23);
      m_oWorkerBaseTester.MakeSureIsProportion(fValue);
      fail("MakeSureIsProportion failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureIsProportion exception processing testing succeeded.");
  }

  /**
   * Tests to make sure a ModelFloat value is greater than a given value
   */
  public void testMakeSureGreaterThanFloat() {
    ModelFloat fValue = new ModelFloat( (float) 0.45,
                                       "ModelFloat MakeSureGreaterThan test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureGreaterThan(fValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureGreaterThanFloat testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureGreaterThanFloat normal processing testing succeeded.");

    //Exception processing - less than the value
    try {
      fValue.SetValue( (float) - 0.23);
      m_oWorkerBaseTester.MakeSureGreaterThan(fValue, 0);
      fail("MakeSureGreaterThanFloat failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }
    
    //Exception processing - equal to the value
    try {
      fValue.SetValue( (float) 0);
      m_oWorkerBaseTester.MakeSureGreaterThan(fValue, 0);
      fail("MakeSureGreaterThanFloat failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureGreaterThanFloat exception processing testing succeeded.");
  }
  
  /**
   * Tests to make sure a ModelInt value is greater than a given value
   */
  public void testMakeSureGreaterThanInt() {
    ModelInt iValue = new ModelInt( 2,
                                       "ModelInt MakeSureGreaterThan test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureGreaterThan(iValue, -1);
    }
    catch (ModelException oErr) {
      fail("MakeSureGreaterThanInt testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureGreaterThanInt normal processing testing succeeded.");

    //Exception processing - less than the value
    try {
      iValue.SetValue(-5);
      m_oWorkerBaseTester.MakeSureGreaterThan(iValue, -1);
      fail("MakeSureGreaterThanInt failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }
    
    //Exception processing - equal to the value
    try {
      iValue.SetValue(-1);
      m_oWorkerBaseTester.MakeSureGreaterThan(iValue, -1);
      fail("MakeSureGreaterThanInt failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureGreaterThanInt exception processing testing succeeded.");
  }
  
  /**
   * Tests to make sure a ModelFloat value is greater than or equal to a 
   * given value
   */
  public void testMakeSureGreaterThanEqualToFloat() {
    ModelFloat fValue = new ModelFloat( (float) 0.45,
                                       "ModelFloat MakeSureGreaterThanEqualTo" +
                                       " test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureGreaterThanEqualTo(fValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureGreaterThanEqualToFloat testing failed with message " +
           oErr.getMessage());
    }
    
    //Normal processing - equal to
    try {
    	fValue.SetValue( (float) 0);
      m_oWorkerBaseTester.MakeSureGreaterThanEqualTo(fValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureGreaterThanEqualToFloat testing failed with message " +
           oErr.getMessage());
    }
    
    System.out.println(
        "MakeSureGreaterThanEqualToFloat normal processing testing succeeded.");

    //Exception processing - less than the value
    try {
      fValue.SetValue( (float) - 0.23);
      m_oWorkerBaseTester.MakeSureGreaterThanEqualTo(fValue, 0);
      fail("MakeSureGreaterThanEqualToFloat failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureGreaterThanEqualToFloat exception processing testing succeeded.");
  }
  
  /**
   * Tests to make sure a ModelInt value is greater than or equal to a given 
   * value
   */
  public void testMakeSureGreaterThanEqualToInt() {
    ModelInt iValue = new ModelInt( 2,
                                       "ModelInt MakeSureGreaterThanEqualTo test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureGreaterThanEqualTo(iValue, -1);
    }
    catch (ModelException oErr) {
      fail("MakeSureGreaterThanEqualToInt testing failed with message " +
           oErr.getMessage());
    }
    
    //Normal processing - equal to
    try {
    	iValue.SetValue(-1);
      m_oWorkerBaseTester.MakeSureGreaterThanEqualTo(iValue, -1);
    }
    catch (ModelException oErr) {
      fail("MakeSureGreaterThanEqualToInt testing failed with message " +
           oErr.getMessage());
    }
    
    System.out.println(
        "MakeSureGreaterThanEqualToInt normal processing testing succeeded.");

    //Exception processing - less than the value
    try {
      iValue.SetValue(-5);
      m_oWorkerBaseTester.MakeSureGreaterThanEqualTo(iValue, -1);
      fail("MakeSureGreaterThanEqualToInt failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }
    
    System.out.println(
        "MakeSureGreaterThanEqualToInt exception processing testing succeeded.");
  }
  
  /**
   * Tests to make sure a ModelFloat value is less than a given value
   */
  public void testMakeSureLessThanFloat() {
    ModelFloat fValue = new ModelFloat( (float) -0.45,
                                       "ModelFloat MakeSureLessThan test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureLessThan(fValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureLessThanFloat testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureLessThanFloat normal processing testing succeeded.");

    //Exception processing - greater than the value
    try {
      fValue.SetValue( (float) 0.23);
      m_oWorkerBaseTester.MakeSureLessThan(fValue, 0);
      fail("MakeSureLessThanFloat failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }
    
    //Exception processing - equal to the value
    try {
      fValue.SetValue( (float) 0);
      m_oWorkerBaseTester.MakeSureLessThan(fValue, 0);
      fail("MakeSureLessThanFloat failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureLessThanFloat exception processing testing succeeded.");
  }
  
  /**
   * Tests to make sure a ModelInt value is less than a given value
   */
  public void testMakeSureLessThanInt() {
    ModelInt iValue = new ModelInt( -2,
                                       "ModelInt MakeSureLessThan test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureLessThan(iValue, -1);
    }
    catch (ModelException oErr) {
      fail("MakeSureLessThanInt testing failed with message " +
           oErr.getMessage());
    }
    System.out.println(
        "MakeSureLessThanInt normal processing testing succeeded.");

    //Exception processing - greater than the value
    try {
      iValue.SetValue(5);
      m_oWorkerBaseTester.MakeSureLessThan(iValue, -1);
      fail("MakeSureLessThanInt failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }
    
    //Exception processing - equal to the value
    try {
      iValue.SetValue(-1);
      m_oWorkerBaseTester.MakeSureLessThan(iValue, -1);
      fail("MakeSureLessThanInt failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureLessThanInt exception processing testing succeeded.");
  }
  
  /**
   * Tests to make sure a ModelFloat value is less than or equal to a 
   * given value
   */
  public void testMakeSureLessThanEqualToFloat() {
    ModelFloat fValue = new ModelFloat( (float) -0.45,
                                       "ModelFloat MakeSureLessThanEqualTo" +
                                       " test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureLessThanEqualTo(fValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureLessThanEqualToFloat testing failed with message " +
           oErr.getMessage());
    }
    
    //Normal processing - equal to
    try {
    	fValue.SetValue( (float) 0);
      m_oWorkerBaseTester.MakeSureLessThanEqualTo(fValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureLessThanEqualToFloat testing failed with message " +
           oErr.getMessage());
    }
    
    System.out.println(
        "MakeSureLessThanEqualToFloat normal processing testing succeeded.");

    //Exception processing - greater than the value
    try {
      fValue.SetValue( (float) 0.23);
      m_oWorkerBaseTester.MakeSureLessThanEqualTo(fValue, 0);
      fail("MakeSureLessThanEqualToFloat failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureLessThanEqualToFloat exception processing testing succeeded.");
  }
  
  /**
   * Tests to make sure a ModelInt value is less than or equal to a given 
   * value
   */
  public void testMakeSureLessThanEqualToInt() {
    ModelInt iValue = new ModelInt( -2,
                                       "ModelInt MakeSureLessThanEqualTo test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureLessThanEqualTo(iValue, -1);
    }
    catch (ModelException oErr) {
      fail("MakeSureLessThanEqualToInt testing failed with message " +
           oErr.getMessage());
    }
    
    //Normal processing - equal to
    try {
    	iValue.SetValue(-1);
      m_oWorkerBaseTester.MakeSureLessThanEqualTo(iValue, -1);
    }
    catch (ModelException oErr) {
      fail("MakeSureLessThanEqualToInt testing failed with message " +
           oErr.getMessage());
    }
    
    System.out.println(
        "MakeSureLessThanEqualToInt normal processing testing succeeded.");

    //Exception processing - greater than the value
    try {
      iValue.SetValue(5);
      m_oWorkerBaseTester.MakeSureLessThanEqualTo(iValue, -1);
      fail("MakeSureLessThanEqualToInt failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }
    
    System.out.println(
        "MakeSureLessThanEqualToInt exception processing testing succeeded.");
  }
  
  /**
   * Tests to make sure a ModelFloat value is not equal to another value
   */
  public void testMakeSureNotEqualToFloat() {
    ModelFloat fValue = new ModelFloat( (float) 0.45,
                                       "ModelFloat MakeSureNotEqualTo test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureNotEqualTo(fValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureNotEqualToFloat testing failed with message " +
           oErr.getMessage());
    }

    fValue.SetValue((float)-0.2);
    try {
      m_oWorkerBaseTester.MakeSureNotEqualTo(fValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureNotEqualToFloat testing failed with message " +
           oErr.getMessage());
    }

    System.out.println(
        "MakeSureNotEqualToFloat normal processing testing succeeded.");

    //Exception processing
    try {
      fValue.SetValue( (float) 0);
      m_oWorkerBaseTester.MakeSureNotEqualTo(fValue, 0);
      fail("MakeSureNotEqualToFloat failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureNotEqualToFloat exception processing testing succeeded.");
  }
  
  /**
   * Tests to make sure a ModelInt value is not equal to another value
   */
  public void testMakeSureNotEqualToInt() {
    ModelInt iValue = new ModelInt( 4, "ModelInt MakeSureNotEqualTo test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureNotEqualTo(iValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureNotEqualToInt testing failed with message " +
           oErr.getMessage());
    }

    iValue.SetValue(-2);
    try {
      m_oWorkerBaseTester.MakeSureNotEqualTo(iValue, 0);
    }
    catch (ModelException oErr) {
      fail("MakeSureNotEqualToInt testing failed with message " +
           oErr.getMessage());
    }

    System.out.println(
        "MakeSureNotEqualToInt normal processing testing succeeded.");

    //Exception processing
    try {
    	iValue.SetValue( 0);
      m_oWorkerBaseTester.MakeSureNotEqualTo(iValue, 0);
      fail("MakeSureNotEqualToInt failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureNotEqualToInt exception processing testing succeeded.");
  }

  /**
   * Tests change of species
   */
  public void testChangeOfSpecies() {
    try {

      //Old number of species
      int iOldNumberOfSpecies = 4;

      //New species indexer
      int[] p_oIndexer = new int[5];
      p_oIndexer[0] = -1; //First species in new list is new
      p_oIndexer[1] = 1; //Second species is the same from old to new
      p_oIndexer[2] = 3; //Third species in new list is same as fourth in old
      p_oIndexer[3] = 0; //Fourth species in new list is same as first in old
      p_oIndexer[4] = -1; //Fifth species in new list is old

      ModelVector
          //Should not get touched
          p_oNonSpeciesSpecificTooShort = new ModelVector("Too short",
          "", "", 0, ModelVector.FLOAT),
          //Should not get touched - right length but not allowed
          p_oNonSpeciesSpecific = new ModelVector("Too long", "", "", 0,
          ModelVector.STRING),
          //Species-specific set of strings
          p_oStringSpeciesSpecific = new ModelVector("String", "", "", 0,
          ModelVector.STRING),
          //Species-specific set of floats
          p_oFloatSpeciesSpecific = new ModelVector("Float", "", "", 0,
          ModelVector.FLOAT),
          //Species-specific set of ModelEnums
          p_oModelEnumSpeciesSpecific = new ModelVector("ModelEnum", "", "", 0,
          ModelVector.MODEL_ENUM),
          //Species-specific set of integers
          p_oIntegerSpeciesSpecific = new ModelVector("ModelInteger", "", "", 0,
          ModelVector.INTEGER);

      p_oNonSpeciesSpecificTooShort.SetIsSpeciesSpecific(false);
      p_oNonSpeciesSpecific.SetIsSpeciesSpecific(false);

      //Give all the vectors data
      p_oNonSpeciesSpecificTooShort.GetValue().add(new Float(1));
      p_oNonSpeciesSpecificTooShort.GetValue().add(new Float(2));

      p_oNonSpeciesSpecific.GetValue().add("test 1");
      p_oNonSpeciesSpecific.GetValue().add("test 2");
      p_oNonSpeciesSpecific.GetValue().add("test 3");
      p_oNonSpeciesSpecific.GetValue().add("test 4");

      p_oStringSpeciesSpecific.GetValue().add("species 0");
      p_oStringSpeciesSpecific.GetValue().add("species 1");
      p_oStringSpeciesSpecific.GetValue().add("species 2");
      p_oStringSpeciesSpecific.GetValue().add("species 3");

      p_oFloatSpeciesSpecific.GetValue().add(new Float(0));
      p_oFloatSpeciesSpecific.GetValue().add(new Float(1));
      p_oFloatSpeciesSpecific.GetValue().add(new Float(2));
      p_oFloatSpeciesSpecific.GetValue().add(new Float(3));

      p_oModelEnumSpeciesSpecific.GetValue().add(new ModelEnum(null, null,
          "enum 0", ""));
      p_oModelEnumSpeciesSpecific.GetValue().add(new ModelEnum(null, null,
          "enum 1", ""));
      p_oModelEnumSpeciesSpecific.GetValue().add(new ModelEnum(null, null,
          "enum 2", ""));
      p_oModelEnumSpeciesSpecific.GetValue().add(new ModelEnum(null, null,
          "enum 3", ""));

      p_oIntegerSpeciesSpecific.GetValue().add(new Integer(0));
      p_oIntegerSpeciesSpecific.GetValue().add(new Integer(1));
      p_oIntegerSpeciesSpecific.GetValue().add(new Integer(2));
      p_oIntegerSpeciesSpecific.GetValue().add(new Integer(3));

      //Put the data in the test object
      WorkerBase oTest = new WorkerTester();
      //ModelInt - should be ignored
      oTest.mp_oAllData.add(new ModelInt(1, "test", "test"));
      oTest.mp_oAllData.add(p_oIntegerSpeciesSpecific);
      oTest.mp_oAllData.add(p_oNonSpeciesSpecificTooShort);
      oTest.mp_oAllData.add(p_oStringSpeciesSpecific);
      oTest.mp_oAllData.add(p_oNonSpeciesSpecific);
      oTest.mp_oAllData.add(p_oModelEnumSpeciesSpecific);
      oTest.mp_oAllData.add(p_oFloatSpeciesSpecific);

      oTest.ChangeOfSpecies(iOldNumberOfSpecies, p_oIndexer, null);

      //Validate each one
      assertEquals(2, p_oNonSpeciesSpecificTooShort.GetValue().size());
      Float fTest = (Float) p_oNonSpeciesSpecificTooShort.GetValue().elementAt(
          0);
      assertEquals(new Float(1), fTest);
      fTest = (Float) p_oNonSpeciesSpecificTooShort.GetValue().elementAt(1);
      assertEquals(new Float(2), fTest);

      assertEquals(4, p_oNonSpeciesSpecific.GetValue().size());
      String sTest = (String) p_oNonSpeciesSpecific.GetValue().elementAt(0);
      assertEquals("test 1", sTest);
      sTest = (String) p_oNonSpeciesSpecific.GetValue().elementAt(1);
      assertEquals("test 2", sTest);
      sTest = (String) p_oNonSpeciesSpecific.GetValue().elementAt(2);
      assertEquals("test 3", sTest);
      sTest = (String) p_oNonSpeciesSpecific.GetValue().elementAt(3);
      assertEquals("test 4", sTest);

      assertEquals(5, p_oStringSpeciesSpecific.GetValue().size());
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(0);
      assertEquals(null, sTest);
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(1);
      assertEquals("species 1", sTest);
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(2);
      assertEquals("species 3", sTest);
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(3);
      assertEquals("species 0", sTest);
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(4);
      assertEquals(null, sTest);

      assertEquals(5, p_oFloatSpeciesSpecific.GetValue().size());
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(0);
      assertEquals(null, fTest);
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(1);
      assertEquals(new Float(1), fTest);
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(2);
      assertEquals(new Float(3), fTest);
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(3);
      assertEquals(new Float(0), fTest);
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(4);
      assertEquals(null, fTest);

      assertEquals(5, p_oModelEnumSpeciesSpecific.GetValue().size());
      ModelEnum oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().
          elementAt(0);
      assertEquals(null, fTest);
      oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().elementAt(
          1);
      assertTrue(oEnumTest.equals(new ModelEnum(null, null, "enum 1", "")));
      oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().elementAt(
          2);
      assertTrue(oEnumTest.equals(new ModelEnum(null, null, "enum 3", "")));
      oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().elementAt(
          3);
      assertTrue(oEnumTest.equals(new ModelEnum(null, null, "enum 0", "")));
      oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().elementAt(
          4);
      assertEquals(null, fTest);

      assertEquals(5, p_oIntegerSpeciesSpecific.GetValue().size());
      Integer iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(
          0);
      assertEquals(null, iTest);
      iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(1);
      assertEquals(new Integer(1), iTest);
      iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(2);
      assertEquals(new Integer(3), iTest);
      iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(3);
      assertEquals(new Integer(0), iTest);
      iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(4);
      assertEquals(null, iTest);

      ////////////////////////////////////////////
      // Testing with some real data
      ////////////////////////////////////////////
      String sFileName = null;
      try {
        GUIManager oManager = new GUIManager(null);
        sFileName = WriteXMLFile1();
        oManager.InputXMLParameterFile(sFileName);

        String[] sNewSpecies = new String[] {
            "Test 1",
            "TSCA",
            "QURU",
            "PRSE",
            "PIST",
            "Test 2",
            "FRAM",
            "FAGR",
            "BEAL",
            "ACSA",
            "ACRU"
        };
        oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);

        LightBehaviors oLight = oManager.GetLightBehaviors();
        ModelVector p_oLEC = oLight.mp_fLightTransmissionCoefficient;
        assertEquals(11, p_oLEC.GetValue().size());
        //Test 1
        fTest = (Float) p_oLEC.GetValue().elementAt(0);
        assertEquals(null, fTest);
        //TSCA
        fTest = (Float) p_oLEC.GetValue().elementAt(1);
        assertEquals(new Float(0.5), fTest);
        //QURU
        fTest = (Float) p_oLEC.GetValue().elementAt(2);
        assertEquals(new Float(0.9), fTest);
        //PRSE
        fTest = (Float) p_oLEC.GetValue().elementAt(3);
        assertEquals(new Float(0.8), fTest);
        //PIST
        fTest = (Float) p_oLEC.GetValue().elementAt(4);
        assertEquals(new Float(0.7), fTest);
        //Test 2
        fTest = (Float) p_oLEC.GetValue().elementAt(5);
        assertEquals(null, fTest);
        //FRAM
        fTest = (Float) p_oLEC.GetValue().elementAt(6);
        assertEquals(new Float(0.6), fTest);
        //FAGR
        fTest = (Float) p_oLEC.GetValue().elementAt(7);
        assertEquals(new Float(0.4), fTest);
        //BEAL
        fTest = (Float) p_oLEC.GetValue().elementAt(8);
        assertEquals(new Float(0.3), fTest);
        //ACSA
        fTest = (Float) p_oLEC.GetValue().elementAt(9);
        assertEquals(new Float(0.2), fTest);
        //ACRU
        fTest = (Float) p_oLEC.GetValue().elementAt(10);
        assertEquals(new Float(0.1), fTest);

      } catch (ModelException oErr) {
        fail("ChangeOfSpecies testing failed with message " + oErr.getMessage());
      } finally {
        new File(sFileName).delete();
      }


      System.out.println("ChangeOfSpecies testing succeeded.");
    } catch (ModelException oErr) {
      fail("ChangeOfSpecies testing failed with message " + oErr.getMessage());
    }
  }


  /**
     * Tests change of species
     */
    public void testCopySpecies() {
        String sFileName = null;
        try {
          GUIManager oManager = new GUIManager(null);
          sFileName = WriteXMLFile1();
          Float fTest;
          oManager.InputXMLParameterFile(sFileName);

          //Add a new species at the end
          String[] sNewSpecies = new String[] {
              "ACRU",
              "ACSA",
              "BEAL",
              "FAGR",
              "TSCA",
              "FRAM",
              "PIST",
              "PRSE",
              "QURU",
              "Test 1"};
          oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
          //Copy BEAL to the last one
          oManager.GetTreePopulation().DoCopySpecies("BEAL", "Test_1");

          //Test a float value
          LightBehaviors oLight = oManager.GetLightBehaviors();
          ModelVector p_oLEC = oLight.mp_fLightTransmissionCoefficient;
          assertEquals(10, p_oLEC.GetValue().size());
          //ACRU
          fTest = (Float) p_oLEC.GetValue().elementAt(0);
          assertEquals(new Float(0.1), fTest);
          //ACSA
          fTest = (Float) p_oLEC.GetValue().elementAt(1);
          assertEquals(new Float(0.2), fTest);
          //BEAL
          fTest = (Float) p_oLEC.GetValue().elementAt(2);
          assertEquals(new Float(0.3), fTest);
          //FAGR
          fTest = (Float) p_oLEC.GetValue().elementAt(3);
          assertEquals(new Float(0.4), fTest);
          //TSCA
          fTest = (Float) p_oLEC.GetValue().elementAt(4);
          assertEquals(new Float(0.5), fTest);
          //FRAM
          fTest = (Float) p_oLEC.GetValue().elementAt(5);
          assertEquals(new Float(0.6), fTest);
          //PIST
          fTest = (Float) p_oLEC.GetValue().elementAt(6);
          assertEquals(new Float(0.7), fTest);
          //PRSE
          fTest = (Float) p_oLEC.GetValue().elementAt(7);
          assertEquals(new Float(0.8), fTest);
          //QURU
          fTest = (Float) p_oLEC.GetValue().elementAt(8);
          assertEquals(new Float(0.9), fTest);
          //Test 1
          fTest = (Float) p_oLEC.GetValue().elementAt(9);
          assertEquals(new Float(0.3), fTest);

          //Test a ModelEnum
          Allometry oAllom = oManager.GetAllometry();
          ModelVector p_oWhatAHD = oAllom.mp_iWhatAdultHDFunction;
          ModelEnum oEnum;
          assertEquals(10, p_oWhatAHD.GetValue().size());
          //ACRU
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(0);
          assertEquals(0, oEnum.GetValue());
          //ACSA
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(1);
          assertEquals(0, oEnum.GetValue());
          //BEAL
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(2);
          assertEquals(1, oEnum.GetValue());
          //FAGR
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(3);
          assertEquals(0, oEnum.GetValue());
          //TSCA
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(4);
          assertEquals(0, oEnum.GetValue());
          //FRAM
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(5);
          assertEquals(0, oEnum.GetValue());
          //PIST
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(6);
          assertEquals(0, oEnum.GetValue());
          //PRSE
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(7);
          assertEquals(0, oEnum.GetValue());
          //QURU
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(8);
          assertEquals(0, oEnum.GetValue());
          //Test 1
          oEnum = (ModelEnum) p_oWhatAHD.GetValue().elementAt(9);
          assertEquals(1, oEnum.GetValue());



        System.out.println("CopySpecies testing succeeded.");
      } catch (ModelException oErr) {
        fail("ChangeOfSpecies testing failed with message " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
    }

  /**
   * This writes an XML file to test species changing.  This file contains
   * light information.
   * @return Name of file.
   * @throws ModelException if there is an IO problem.
   */
  protected String WriteXMLFile1() throws ModelException {
    try {
      String sFileName = "\\loratestxml1.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<paramFile fileCode=\"06050101\">");
      oOut.write("<plot>");
      oOut.write("<timesteps>400</timesteps>");
      oOut.write("<randomSeed>0</randomSeed>");
      oOut.write("<yearsPerTimestep>5.0</yearsPerTimestep>");
      oOut.write("<plot_lenX>500.0</plot_lenX>");
      oOut.write("<plot_lenY>500.0</plot_lenY>");
      oOut.write("<plot_latitude>41.92</plot_latitude>");
      oOut.write("</plot>");
      oOut.write("<trees>");
      oOut.write("<tr_speciesList>");
      oOut.write("<tr_species speciesName=\"ACRU\"/>");
      oOut.write("<tr_species speciesName=\"ACSA\"/>");
      oOut.write("<tr_species speciesName=\"BEAL\"/>");
      oOut.write("<tr_species speciesName=\"FAGR\"/>");
      oOut.write("<tr_species speciesName=\"TSCA\"/>");
      oOut.write("<tr_species speciesName=\"FRAM\"/>");
      oOut.write("<tr_species speciesName=\"PIST\"/>");
      oOut.write("<tr_species speciesName=\"PRSE\"/>");
      oOut.write("<tr_species speciesName=\"QURU\"/>");
      oOut.write("</tr_speciesList>");
      oOut.write("<tr_seedDiam10Cm>0.2</tr_seedDiam10Cm>");
      oOut.write("</trees>");
      oOut.write("<behaviorList>");
      oOut.write("<behavior>");
      oOut.write("<behaviorName>quadratlight</behaviorName>");
      oOut.write("<version>1.0</version>");
      oOut.write("<applyTo species=\"ACRU\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"ACSA\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"BEAL\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"FAGR\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"TSCA\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"FRAM\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"PIST\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"PRSE\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"QURU\" type=\"Seedling\"/>");
      oOut.write("</behavior>");
      oOut.write("<behavior>");
      oOut.write("<behaviorName>constradialgrowth</behaviorName>");
      oOut.write("<version>1.0</version>");
      oOut.write("<applyTo species=\"PIST\" type=\"Adult\"/>");
      oOut.write("<applyTo species=\"PRSE\" type=\"Adult\"/>");
      oOut.write("<applyTo species=\"QURU\" type=\"Adult\"/>");
      oOut.write("</behavior>");
      oOut.write("</behaviorList>");
      oOut.write("<allometry>");
      oOut.write("<tr_whatAdultHeightDiam>");
      oOut.write("<tr_wahdVal species=\"ACRU\">0</tr_wahdVal>");
      oOut.write("<tr_wahdVal species=\"ACSA\">0</tr_wahdVal>");
      oOut.write("<tr_wahdVal species=\"BEAL\">1</tr_wahdVal>");
      oOut.write("<tr_wahdVal species=\"FAGR\">0</tr_wahdVal>");
      oOut.write("<tr_wahdVal species=\"TSCA\">0</tr_wahdVal>");
      oOut.write("<tr_wahdVal species=\"FRAM\">0</tr_wahdVal>");
      oOut.write("<tr_wahdVal species=\"PIST\">0</tr_wahdVal>");
      oOut.write("<tr_wahdVal species=\"PRSE\">0</tr_wahdVal>");
      oOut.write("<tr_wahdVal species=\"QURU\">0</tr_wahdVal>");
      oOut.write("</tr_whatAdultHeightDiam>");
      oOut.write("</allometry>");
      oOut.write("<gliLight>");
      oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
      oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
      oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
      oOut.write("<li_julianDayGrowthStarts>120</li_julianDayGrowthStarts>");
      oOut.write("<li_julianDayGrowthEnds>270</li_julianDayGrowthEnds>");
      oOut.write("<li_minSunAngle>0.779</li_minSunAngle>");
      oOut.write("<li_numAltGrids>12</li_numAltGrids>");
      oOut.write("<li_numAziGrids>18</li_numAziGrids>");
      oOut.write("<li_lightExtinctionCoefficient>");
      oOut.write("<li_lecVal species=\"ACRU\">0.1</li_lecVal>");
      oOut.write("<li_lecVal species=\"ACSA\">0.2</li_lecVal>");
      oOut.write("<li_lecVal species=\"BEAL\">0.3</li_lecVal>");
      oOut.write("<li_lecVal species=\"FAGR\">0.4</li_lecVal>");
      oOut.write("<li_lecVal species=\"TSCA\">0.5</li_lecVal>");
      oOut.write("<li_lecVal species=\"FRAM\">0.6</li_lecVal>");
      oOut.write("<li_lecVal species=\"PIST\">0.7</li_lecVal>");
      oOut.write("<li_lecVal species=\"PRSE\">0.8</li_lecVal>");
      oOut.write("<li_lecVal species=\"QURU\">0.9</li_lecVal>");
      oOut.write("</li_lightExtinctionCoefficient>");
      oOut.write("<li_snagAgeClass1>0</li_snagAgeClass1>");
      oOut.write("<li_snagAgeClass2>0</li_snagAgeClass2>");
      oOut.write("</gliLight>");
      oOut.write("<growth>");
      oOut.write("<gr_adultConstRadialInc>");
      oOut.write("<gr_acriVal species=\"PIST\">1.14</gr_acriVal>");
      oOut.write("<gr_acriVal species=\"PRSE\">2.47</gr_acriVal>");
      oOut.write("<gr_acriVal species=\"QURU\">0.72</gr_acriVal>");
      oOut.write("</gr_adultConstRadialInc>");
      oOut.write("</growth>");
      oOut.write("</paramFile>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }

}

/**
 * Test class for WorkerBase.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
class WorkerTester
    extends WorkerBase {

  /**
   * Constructor.
   */
  WorkerTester() {
    super(null, "Test", null);
  }

  /**
   * Does nothing
   * @param oPop TreePopulation Ignored
   */
  public void DoSetup(TreePopulation oPop) {;}

  /**
   * Overridden abstract function
   * @param oPop Not used
   * @throws ModelException never
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {};

  /**
   * Overridden abstract function
   * @param oPop Not used
   * @param out Not used
   * @throws ModelException never
   */
  public void WriteXML(java.io.FileWriter out, TreePopulation oPop) throws
      ModelException {};
}
